@@ -329,7 +329,7 @@ soup_cookie_jar_get_cookies (SoupCookieJar *jar, SoupURI *uri,
  *
  * @cookie will be 'stolen' by the jar, so don't free it afterwards.
  *
- * Since: 2.24
+ * Since: 2.26
  **/
 void
 soup_cookie_jar_add_cookie (SoupCookieJar *jar, SoupCookie *cookie)
@@ -345,7 +345,8 @@ soup_cookie_jar_add_cookie (SoupCookieJar *jar, SoupCookie *cookie)
 	old_cookies = g_hash_table_lookup (priv->domains, cookie->domain);
 	for (oc = old_cookies; oc; oc = oc->next) {
 		old_cookie = oc->data;
-		if (!strcmp (cookie->name, old_cookie->name)) {
+		if (!strcmp (cookie->name, old_cookie->name) &&
+		    !g_strcmp0 (cookie->path, old_cookie->path)) {
 			if (cookie->expires && soup_date_is_past (cookie->expires)) {
 				/* The new cookie has an expired date,
 				 * this is the way the the server has
@@ -469,7 +470,7 @@ request_unqueued (SoupSessionFeature *feature, SoupSession *session,
  *
  * Return value: a #GSList with all the cookies in the @jar.
  *
- * Since: 2.24
+ * Since: 2.26
  **/
 GSList *
 soup_cookie_jar_all_cookies (SoupCookieJar *jar)
@@ -501,7 +502,7 @@ soup_cookie_jar_all_cookies (SoupCookieJar *jar)
  *
  * Deletes @cookie from @jar, emitting the 'changed' signal.
  *
- * Since: 2.24
+ * Since: 2.26
  **/
 void
 soup_cookie_jar_delete_cookie (SoupCookieJar *jar,
