@@ -9,12 +9,15 @@
 #include <config.h>
 #endif
 
+#define LIBSOUP_I_HAVE_READ_BUG_594377_AND_KNOW_SOUP_PASSWORD_MANAGER_MIGHT_GO_AWAY
+
 #include "soup-address.h"
 #include "soup-session-async.h"
 #include "soup-session-private.h"
 #include "soup-address.h"
 #include "soup-message-private.h"
 #include "soup-misc.h"
+#include "soup-password-manager.h"
 #include "soup-proxy-uri-resolver.h"
 #include "soup-uri.h"
 
@@ -34,6 +37,9 @@ static void  queue_message   (SoupSession *session, SoupMessage *req,
 			      SoupSessionCallback callback, gpointer user_data);
 static guint send_message    (SoupSession *session, SoupMessage *req);
 
+static void  auth_required   (SoupSession *session, SoupMessage *msg,
+			      SoupAuth *auth, gboolean retrying);
+
 G_DEFINE_TYPE (SoupSessionAsync, soup_session_async, SOUP_TYPE_SESSION)
 
 typedef struct {
@@ -69,6 +75,7 @@ soup_session_async_class_init (SoupSessionAsyncClass *soup_session_async_class)
 	/* virtual method override */
 	session_class->queue_message = queue_message;
 	session_class->send_message = send_message;
+	session_class->auth_required = auth_required;
 
 	object_class->finalize = finalize;
 }
@@ -296,13 +303,13 @@ run_queue (SoupSessionAsync *sa)
 	SoupSession *session = SOUP_SESSION (sa);
 	SoupMessageQueue *queue = soup_session_get_queue (session);
 	SoupMessageQueueItem *item;
-	SoupProxyURIResolver *proxy_resolver =
-		soup_session_get_proxy_resolver (session);
+	SoupProxyURIResolver *proxy_resolver;
 	SoupMessage *msg;
-	SoupMessageIOStatus cur_io_status = SOUP_MESSAGE_IO_STATUS_CONNECTING;
 	SoupConnection *conn;
 	gboolean try_pruning = TRUE, should_prune = FALSE;
 
+	soup_session_cleanup_connections (session, FALSE);
+
  try_again:
 	for (item = soup_message_queue_first (queue);
 	     item && !should_prune;
@@ -313,13 +320,16 @@ run_queue (SoupSessionAsync *sa)
 		if (msg->method == SOUP_METHOD_CONNECT)
 			continue;
 
-		if (soup_message_get_io_status (msg) != cur_io_status ||
-		    soup_message_io_in_progress (msg))
+		if (soup_message_io_in_progress (msg))
 			continue;
 
-		if (proxy_resolver && !item->resolved_proxy_addr) {
-			resolve_proxy_addr (item, proxy_resolver);
-			continue;
+		if (!item->resolved_proxy_addr) {
+			proxy_resolver = (SoupProxyURIResolver *)soup_session_get_feature_for_message (session, SOUP_TYPE_PROXY_URI_RESOLVER, msg);
+			if (proxy_resolver) {
+				resolve_proxy_addr (item, proxy_resolver);
+				continue;
+			} else
+				item->resolved_proxy_addr = TRUE;
 		}
 
 		conn = soup_session_get_connection (session, item,
@@ -336,17 +346,12 @@ run_queue (SoupSessionAsync *sa)
 	if (item)
 		soup_message_queue_item_unref (item);
 
-	if (cur_io_status == SOUP_MESSAGE_IO_STATUS_CONNECTING) {
-		cur_io_status = SOUP_MESSAGE_IO_STATUS_QUEUED;
-		goto try_again;
-	}
-
 	if (try_pruning && should_prune) {
 		/* There is at least one message in the queue that
 		 * could be sent if we pruned an idle connection from
 		 * some other server.
 		 */
-		if (soup_session_try_prune_connection (session)) {
+		if (soup_session_cleanup_connections (session, TRUE)) {
 			try_pruning = should_prune = FALSE;
 			goto try_again;
 		}
@@ -441,3 +446,36 @@ send_message (SoupSession *session, SoupMessage *req)
 
 	return req->status_code;
 }
+
+static void
+got_passwords (SoupPasswordManager *password_manager, SoupMessage *msg,
+	       SoupAuth *auth, gboolean retrying, gpointer session)
+{
+	soup_session_unpause_message (session, msg);
+	SOUP_SESSION_CLASS (soup_session_async_parent_class)->
+		auth_required (session, msg, auth, retrying);
+	g_object_unref (auth);
+}
+
+static void
+auth_required (SoupSession *session, SoupMessage *msg,
+	       SoupAuth *auth, gboolean retrying)
+{
+	SoupSessionFeature *password_manager;
+
+	password_manager = soup_session_get_feature_for_message (
+		session, SOUP_TYPE_PASSWORD_MANAGER, msg);
+	if (password_manager) {
+		soup_session_pause_message (session, msg);
+		g_object_ref (auth);
+		soup_password_manager_get_passwords_async (
+			SOUP_PASSWORD_MANAGER (password_manager),
+			msg, auth, retrying,
+			soup_session_get_async_context (session),
+			NULL, /* FIXME cancellable */
+			got_passwords, session);
+	} else {
+		SOUP_SESSION_CLASS (soup_session_async_parent_class)->
+			auth_required (session, msg, auth, retrying);
+	}
+}
