@@ -83,12 +83,6 @@ typedef struct {
 } SoupSocketPrivate;
 #define SOUP_SOCKET_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), SOUP_TYPE_SOCKET, SoupSocketPrivate))
 
-#ifdef HAVE_IPV6
-#define soup_sockaddr_max sockaddr_in6
-#else
-#define soup_sockaddr_max sockaddr_in
-#endif
-
 static void set_property (GObject *object, guint prop_id,
 			  const GValue *value, GParamSpec *pspec);
 static void get_property (GObject *object, guint prop_id,
@@ -844,7 +838,7 @@ listen_watch (GIOChannel* iochannel, GIOCondition condition, gpointer data)
 {
 	SoupSocket *sock = data, *new;
 	SoupSocketPrivate *priv = SOUP_SOCKET_GET_PRIVATE (sock), *new_priv;
-	struct soup_sockaddr_max sa;
+	struct sockaddr_storage sa;
 	int sa_len, sockfd;
 
 	if (condition & (G_IO_HUP | G_IO_ERR)) {
@@ -1111,7 +1105,7 @@ soup_socket_get_local_address (SoupSocket *sock)
 
 	g_mutex_lock (priv->addrlock);
 	if (!priv->local_addr) {
-		struct soup_sockaddr_max bound_sa;
+		struct sockaddr_storage bound_sa;
 		int sa_len;
 
 		sa_len = sizeof (bound_sa);
@@ -1141,7 +1135,7 @@ soup_socket_get_remote_address (SoupSocket *sock)
 
 	g_mutex_lock (priv->addrlock);
 	if (!priv->remote_addr) {
-		struct soup_sockaddr_max bound_sa;
+		struct sockaddr_storage bound_sa;
 		int sa_len;
 
 		sa_len = sizeof (bound_sa);
@@ -1212,8 +1206,11 @@ read_from_network (SoupSocket *sock, gpointer buffer, gsize len,
 	if (!priv->iochannel)
 		return SOUP_SOCKET_EOF;
 
-	if (priv->timed_out)
+	if (priv->timed_out) {
+		g_set_error_literal (error, G_IO_ERROR, G_IO_ERROR_TIMED_OUT,
+				     "Timed out");
 		return SOUP_SOCKET_ERROR;
+	}
 
 	status = g_io_channel_read_chars (priv->iochannel,
 					  buffer, len, nread, &my_err);
@@ -1236,8 +1233,11 @@ read_from_network (SoupSocket *sock, gpointer buffer, gsize len,
 		 * a socket timeout and should be treated as an error
 		 * condition.
 		 */
-		if (!priv->non_blocking)
+		if (!priv->non_blocking) {
+			g_set_error_literal (error, G_IO_ERROR, G_IO_ERROR_TIMED_OUT,
+					     "Timed out");
 			return SOUP_SOCKET_ERROR;
+		}
 
 		if (!priv->read_src) {
 			priv->read_src =
@@ -1502,6 +1502,8 @@ soup_socket_write (SoupSocket *sock, gconstpointer buffer,
 	}
 	if (priv->timed_out) {
 		g_mutex_unlock (priv->iolock);
+		g_set_error_literal (error, G_IO_ERROR, G_IO_ERROR_TIMED_OUT,
+				     "Timed out");
 		return SOUP_SOCKET_ERROR;
 	}
 	if (priv->write_src) {
@@ -1523,6 +1525,8 @@ soup_socket_write (SoupSocket *sock, gconstpointer buffer,
 	 */
 	if (!priv->non_blocking && status == G_IO_STATUS_AGAIN) {
 		g_mutex_unlock (priv->iolock);
+		g_set_error_literal (error, G_IO_ERROR, G_IO_ERROR_TIMED_OUT,
+				     "Timed out");
 		return SOUP_SOCKET_ERROR;
 	}
 
