@@ -95,7 +95,7 @@ static void append_uri_encoded (GString *str, const char *in, const char *extra_
 static char *uri_decoded_copy (const char *str, int length);
 static char *uri_normalized_copy (const char *str, int length, const char *unescape_extra, gboolean fixup);
 
-const char *_SOUP_URI_SCHEME_HTTP, *_SOUP_URI_SCHEME_HTTPS;
+gpointer _SOUP_URI_SCHEME_HTTP, _SOUP_URI_SCHEME_HTTPS;
 
 static inline const char *
 soup_uri_get_scheme (const char *scheme, int len)
@@ -247,14 +247,12 @@ soup_uri_new_with_base (SoupURI *base, const char *uri_string)
 	/* Find query */
 	question = memchr (uri_string, '?', end - uri_string);
 	if (question) {
-		if (question[1]) {
-			uri->query = uri_normalized_copy (question + 1,
-							  end - (question + 1),
-							  NULL, TRUE);
-			if (!uri->query) {
-				soup_uri_free (uri);
-				return NULL;
-			}
+		uri->query = uri_normalized_copy (question + 1,
+						  end - (question + 1),
+						  NULL, TRUE);
+		if (!uri->query) {
+			soup_uri_free (uri);
+			return NULL;
 		}
 		end = question;
 	}
